"""
PyBatis 메인 클래스의 테스트

README.md에서 제시한 새로운 API를 테스트합니다.
"""

from typing import List, Optional

import pytest

from pybatis import PyBatis
from tests.fixtures import User, MockConnection


class TestPyBatis:
    """PyBatis 클래스의 테스트"""

    @pytest.fixture
    def mock_connection(self):
        """테스트용 연결 픽스처"""
        return MockConnection()

    @pytest.fixture
    def pybatis(self, mock_connection):
        """테스트용 PyBatis 인스턴스"""
        db = PyBatis()
        db._connection = mock_connection  # 테스트용 연결 주입
        return db

    @pytest.mark.asyncio
    async def test_fetch_val(self, pybatis, mock_connection):
        """fetch_val 메서드 테스트"""
        sql = "SELECT COUNT(*) FROM users WHERE is_active = :active"
        params = {"active": True}

        result = await pybatis.fetch_val(sql, params=params)

        assert result == 5
        assert sql in mock_connection.executed_sql
        assert params in mock_connection.executed_params

    @pytest.mark.asyncio
    async def test_fetch_one(self, pybatis, mock_connection):
        """fetch_one 메서드 테스트"""
        sql = "SELECT id, name, email, is_active FROM users WHERE id = :user_id"
        params = {"user_id": 1}

        result = await pybatis.fetch_one(sql, params=params)

        assert result is not None
        assert result["id"] == 1
        assert result["name"] == "테스트사용자"
        assert sql in mock_connection.executed_sql
        assert params in mock_connection.executed_params

    @pytest.mark.asyncio
    async def test_fetch_all(self, pybatis, mock_connection):
        """fetch_all 메서드 테스트"""
        sql = (
            "SELECT id, name, email, is_active FROM users "
            "WHERE is_active = :active_status"
        )
        params = {"active_status": True}

        result = await pybatis.fetch_all(sql, params=params)

        assert len(result) == 2
        assert result[0]["id"] == 1
        assert result[1]["id"] == 2
        assert sql in mock_connection.executed_sql
        assert params in mock_connection.executed_params

    @pytest.mark.asyncio
    async def test_execute(self, pybatis, mock_connection):
        """execute 메서드 테스트"""
        sql = "INSERT INTO users (name, email) VALUES (:name, :email)"
        params = {"name": "새사용자", "email": "new@example.com"}

        result = await pybatis.execute(sql, params=params)

        assert result == 1
        assert sql in mock_connection.executed_sql
        assert params in mock_connection.executed_params


class TestUserRepository:
    """README.md 예시의 UserRepository 테스트"""

    @pytest.fixture
    def mock_connection(self):
        """테스트용 연결 픽스처"""
        return MockConnection()

    @pytest.fixture
    def pybatis(self, mock_connection):
        """테스트용 PyBatis 인스턴스"""
        db = PyBatis()
        db._connection = mock_connection
        return db

    @pytest.fixture
    def user_repository(self, pybatis):
        """테스트용 UserRepository 인스턴스"""
        from tests.fixtures import UserRepository

        return UserRepository(pybatis)

    @pytest.mark.asyncio
    async def test_count_active(self, user_repository):
        """count_active 메서드 테스트"""
        count = await user_repository.count_active(active=True)
        assert count == 5

    @pytest.mark.asyncio
    async def test_get_user_by_id(self, user_repository):
        """get_user_by_id 메서드 테스트"""
        user = await user_repository.get_user_by_id(user_id=1)

        assert isinstance(user, User)
        assert user.id == 1
        assert user.name == "테스트사용자"
        assert user.is_active is True

    @pytest.mark.asyncio
    async def test_get_user_by_id_not_found(self, user_repository):
        """존재하지 않는 사용자 조회 테스트"""
        user = await user_repository.get_user_by_id(user_id=999)
        assert user is None

    @pytest.mark.asyncio
    async def test_get_users_by_activity(self, user_repository):
        """get_users_by_activity 메서드 테스트"""
        users = await user_repository.get_users_by_activity(active_status=True)

        assert len(users) == 2
        assert all(isinstance(user, User) for user in users)
        assert all(user.is_active for user in users)


class TestPyBatisDSN:
    """PyBatis DSN 초기화 테스트"""

    def test_create_with_dsn(self):
        """DSN으로 PyBatis 인스턴스 생성 테스트"""
        dsn = "postgresql://user:pass@localhost:5432/mydb"
        db = PyBatis(dsn=dsn)

        assert db.dsn == dsn
        assert db._connection is None  # 아직 연결되지 않음

    def test_create_without_dsn(self):
        """DSN 없이 PyBatis 인스턴스 생성 테스트"""
        db = PyBatis()

        assert db.dsn is None
        assert db._connection is None



"""
테스트용 픽스처들

README.md 예시에 맞는 UserRepository 등을 포함합니다.
"""

from typing import List, Optional

from pydantic import BaseModel


class MockConnection:
    """테스트용 데이터베이스 연결 모킹"""

    def __init__(self):
        self.executed_sql = []
        self.executed_params = []
        self.mock_data = {}

    async def execute(self, sql: str, params: Optional[dict] = None) -> int:
        """SQL 실행을 모킹합니다."""
        self.executed_sql.append(sql)
        self.executed_params.append(params)
        return 1

    async def fetchval(self, sql: str, params: Optional[dict] = None) -> int:
        """스칼라 값 조회를 모킹합니다."""
        self.executed_sql.append(sql)
        self.executed_params.append(params)

        # COUNT 쿼리 모킹
        if "COUNT" in sql:
            if params and params.get("active"):
                return 5  # 활성 사용자 5명
            return 2  # 비활성 사용자 2명
        return 0

    async def fetchrow(self, sql: str, params: Optional[dict] = None) -> Optional[dict]:
        """하나의 레코드 조회를 모킹합니다."""
        self.executed_sql.append(sql)
        self.executed_params.append(params)

        if params and params.get("user_id") == 1:
            return {
                "id": 1,
                "name": "테스트사용자",
                "email": "test@example.com",
                "is_active": True,
            }
        return None

    async def fetch(self, sql: str, params: Optional[dict] = None) -> List[dict]:
        """모든 레코드 조회를 모킹합니다."""
        self.executed_sql.append(sql)
        self.executed_params.append(params)

        if params and params.get("active_status"):
            return [
                {
                    "id": 1,
                    "name": "활성사용자1",
                    "email": "active1@example.com",
                    "is_active": True,
                },
                {
                    "id": 2,
                    "name": "활성사용자2",
                    "email": "active2@example.com",
                    "is_active": True,
                },
            ]
        else:
            return [
                {
                    "id": 3,
                    "name": "비활성사용자1",
                    "email": "inactive1@example.com",
                    "is_active": False,
                },
            ]

    async def close(self) -> None:
        """연결 닫기를 모킹합니다."""
        pass


class User(BaseModel):
    """사용자 모델"""

    id: int
    name: str
    email: str
    is_active: bool


class UserRepository:
    """README.md 예시의 UserRepository"""

    def __init__(self, db):
        self.db = db

    async def count_active(self, active: bool) -> int:
        """
        활성 사용자 수를 반환합니다.
        """
        sql = "SELECT COUNT(*) FROM users WHERE is_active = :active"
        # fetch_val: 단일 스칼라 값을 바로 리턴
        return await self.db.fetch_val(sql, params={"active": active})

    async def get_user_by_id(self, user_id: int) -> Optional[User]:
        """
        주어진 ID의 사용자 한 명을 User 모델로 반환합니다.
        """
        sql = "SELECT id, name, email, is_active FROM users WHERE id = :user_id"
        row = await self.db.fetch_one(sql, params={"user_id": user_id})

        if row is None:
            return None

        return User(**row)

    async def get_users_by_activity(self, active_status: bool) -> List[User]:
        """
        활성 상태에 따라 사용자 목록을 User 모델 리스트로 반환합니다.
        """
        sql = (
            "SELECT id, name, email, is_active FROM users "
            "WHERE is_active = :active_status"
        )
        rows = await self.db.fetch_all(sql, params={"active_status": active_status})

        return [User(**row) for row in rows]



"""
PyBatis 메인 클래스

README.md에서 제시한 API를 구현합니다.
"""

import logging
import time
from collections import defaultdict
from contextlib import asynccontextmanager
from pathlib import Path
from typing import TYPE_CHECKING, Any, AsyncGenerator, Dict, List, Optional, Union
from urllib.parse import urlparse

if TYPE_CHECKING:
    from .sql_loader import SqlLoader

logger = logging.getLogger(__name__)


class PyBatis:
    """
    pyBatis의 메인 클래스

    DSN을 통해 데이터베이스에 연결하고,
    fetch_val, fetch_one, fetch_all, execute 메서드를 제공합니다.
    """

    def __init__(
        self,
        dsn: Optional[str] = None,
        sql_dir: Optional[Union[str, Path]] = None,
    ):
        """
        PyBatis 인스턴스를 초기화합니다.

        Args:
            dsn: 데이터베이스 연결 문자열 (예: "sqlite:///path/to/db.sqlite")
            sql_dir: SQL 파일들이 있는 디렉토리 경로 (옵션)
        """
        self.dsn = dsn
        self._connection = None
        self.sql_loader: Optional["SqlLoader"] = None
        self._db_type: Optional[str] = None  # 데이터베이스 타입 저장
        self._table_schemas: Dict[str, Dict[str, str]] = {}  # 테이블 스키마 캐시

        # 쿼리 로깅 관련 속성
        self._query_logging_enabled = False
        self._query_logging_level = logging.INFO

        # 쿼리 모니터링 관련 속성
        self._query_monitoring_enabled = False
        self._query_stats = None
        self._slow_query_threshold = 1.0  # 기본 1초

        # SQL 디렉토리가 제공되면 SQL 로더 초기화
        if sql_dir:
            self.set_sql_loader_dir(sql_dir)

    def enable_query_logging(self, level: int = logging.INFO) -> None:
        """
        쿼리 로깅을 활성화합니다.

        Args:
            level: 로깅 레벨 (기본값: logging.INFO)
        """
        self._query_logging_enabled = True
        self._query_logging_level = level
        logger.info("쿼리 로깅이 활성화되었습니다.")

    def disable_query_logging(self) -> None:
        """쿼리 로깅을 비활성화합니다."""
        self._query_logging_enabled = False
        logger.info("쿼리 로깅이 비활성화되었습니다.")

    def enable_query_monitoring(self) -> None:
        """쿼리 모니터링을 활성화합니다."""
        self._query_monitoring_enabled = True
        self._query_stats = {
            "total_queries": 0,
            "method_counts": defaultdict(int),
            "execution_times": [],
            "slow_queries": [],
        }
        logger.info("쿼리 모니터링이 활성화되었습니다.")

    def disable_query_monitoring(self) -> None:
        """쿼리 모니터링을 비활성화합니다."""
        self._query_monitoring_enabled = False
        self._query_stats = None
        logger.info("쿼리 모니터링이 비활성화되었습니다.")

    def get_query_stats(self) -> Optional[Dict[str, Any]]:
        """
        쿼리 통계를 반환합니다.

        Returns:
            쿼리 통계 딕셔너리 또는 None (모니터링이 비활성화된 경우)
        """
        if not self._query_monitoring_enabled or self._query_stats is None:
            return None

        # 통계 복사본 반환 (원본 보호)
        return {
            "total_queries": self._query_stats["total_queries"],
            "method_counts": dict(self._query_stats["method_counts"]),
            "execution_times": self._query_stats["execution_times"].copy(),
            "slow_queries": self._query_stats["slow_queries"].copy(),
            "average_execution_time": (
                sum(self._query_stats["execution_times"]) / len(self._query_stats["execution_times"])
                if self._query_stats["execution_times"] else 0
            ),
        }

    def reset_query_stats(self) -> None:
        """쿼리 통계를 초기화합니다."""
        if self._query_monitoring_enabled and self._query_stats is not None:
            self._query_stats = {
                "total_queries": 0,
                "method_counts": defaultdict(int),
                "execution_times": [],
                "slow_queries": [],
            }
            logger.info("쿼리 통계가 초기화되었습니다.")

    def set_slow_query_threshold(self, threshold: float) -> None:
        """
        느린 쿼리 임계값을 설정합니다.

        Args:
            threshold: 임계값 (초 단위)
        """
        self._slow_query_threshold = threshold
        logger.info(f"느린 쿼리 임계값이 {threshold}초로 설정되었습니다.")

    def _log_query_execution(self, method: str, sql: str, params: Optional[Dict[str, Any]], execution_time: float) -> None:
        """
        쿼리 실행을 로깅합니다.

        Args:
            method: 실행된 메서드명
            sql: 실행된 SQL 문
            params: SQL 파라미터
            execution_time: 실행 시간 (초)
        """
        if self._query_logging_enabled:
            # SQL 문을 한 줄로 정리
            clean_sql = " ".join(sql.split())

            log_message = f"SQL 실행 ({method}): {clean_sql}"
            if params:
                log_message += f", params: {params}"
            log_message += f", 실행 시간: {execution_time:.4f}초"

            logger.log(self._query_logging_level, log_message)

            # 느린 쿼리 경고
            if execution_time > self._slow_query_threshold:
                logger.warning(f"느린 쿼리 감지: {execution_time:.4f}초 (임계값: {self._slow_query_threshold}초) - {clean_sql}")

    def _record_query_stats(self, method: str, sql: str, execution_time: float) -> None:
        """
        쿼리 통계를 기록합니다.

        Args:
            method: 실행된 메서드명
            sql: 실행된 SQL 문
            execution_time: 실행 시간 (초)
        """
        if self._query_monitoring_enabled and self._query_stats is not None:
            self._query_stats["total_queries"] += 1
            self._query_stats["method_counts"][method] += 1
            self._query_stats["execution_times"].append(execution_time)

            # 느린 쿼리 기록
            if execution_time > self._slow_query_threshold:
                clean_sql = " ".join(sql.split())
                self._query_stats["slow_queries"].append({
                    "sql": clean_sql,
                    "execution_time": execution_time,
                    "method": method,
                    "timestamp": time.time(),
                })

    async def _execute_with_monitoring(self, method: str, sql: str, params: Optional[Dict[str, Any]], executor_func) -> Any:
        """
        모니터링과 함께 쿼리를 실행합니다.

        Args:
            method: 실행할 메서드명
            sql: 실행할 SQL 문
            params: SQL 파라미터
            executor_func: 실제 실행할 함수

        Returns:
            쿼리 실행 결과
        """
        start_time = time.time()

        try:
            result = await executor_func()
            return result
        finally:
            execution_time = time.time() - start_time

            # 로깅 및 모니터링
            self._log_query_execution(method, sql, params, execution_time)
            self._record_query_stats(method, sql, execution_time)

    async def _get_table_schema(self, table_name: str) -> Dict[str, str]:
        """
        테이블의 스키마 정보를 가져옵니다.

        Args:
            table_name: 테이블명

        Returns:
            컬럼명 -> 데이터 타입 매핑 딕셔너리
        """
        if table_name in self._table_schemas:
            return self._table_schemas[table_name]

        if self._db_type == "sqlite" and hasattr(self._connection, 'execute'):
            # SQLite PRAGMA를 사용하여 테이블 스키마 정보 가져오기
            async with self._connection.execute(f"PRAGMA table_info({table_name})") as cursor:
                columns = await cursor.fetchall()
                schema = {}
                for col in columns:
                    col_dict = dict(col)
                    col_name = col_dict['name']
                    col_type = col_dict['type'].upper()
                    schema[col_name] = col_type

                self._table_schemas[table_name] = schema
                return schema

        return {}

    async def _extract_table_name_from_sql(self, sql: str) -> Optional[str]:
        """
        SQL 문에서 테이블명을 추출합니다.

        Args:
            sql: SQL 문

        Returns:
            테이블명 또는 None
        """
        import re

        # 간단한 SELECT 문에서 테이블명 추출
        # FROM 절에서 테이블명을 찾는 정규식
        patterns = [
            r'FROM\s+([a-zA-Z_][a-zA-Z0-9_]*)',  # FROM table_name
            r'UPDATE\s+([a-zA-Z_][a-zA-Z0-9_]*)',  # UPDATE table_name
            r'INSERT\s+INTO\s+([a-zA-Z_][a-zA-Z0-9_]*)',  # INSERT INTO table_name
        ]

        sql_upper = sql.upper()
        for pattern in patterns:
            match = re.search(pattern, sql_upper)
            if match:
                return match.group(1).lower()

        return None

    async def _convert_row_data_with_schema(self, row_data: Dict[str, Any], table_name: Optional[str] = None) -> Dict[str, Any]:
        """
        데이터베이스 스키마 정보를 사용하여 데이터 타입을 자동 변환합니다.

        Args:
            row_data: 데이터베이스에서 가져온 행 데이터
            table_name: 테이블명 (옵션)

        Returns:
            변환된 행 데이터
        """
        if not row_data or self._db_type != "sqlite":
            return row_data

        # 테이블 스키마 정보가 있는 경우에만 변환 수행
        if table_name:
            try:
                schema = await self._get_table_schema(table_name)
                converted_data = {}

                for key, value in row_data.items():
                    col_type = schema.get(key, '').upper()

                    # SQLite에서 BOOLEAN 타입으로 정의된 컬럼을 boolean으로 변환
                    if col_type == 'BOOLEAN' and isinstance(value, int) and value in (0, 1):
                        converted_data[key] = bool(value)
                    else:
                        converted_data[key] = value

                return converted_data
            except Exception as e:
                # 스키마 정보를 가져올 수 없는 경우 원본 데이터 반환
                logger.debug(f"스키마 정보를 가져올 수 없습니다: {e}")
                return row_data

        return row_data

    async def _convert_rows_data_with_schema(self, rows_data: List[Dict[str, Any]], table_name: Optional[str] = None) -> List[Dict[str, Any]]:
        """
        스키마 정보를 사용하여 여러 행의 데이터를 변환합니다.

        Args:
            rows_data: 데이터베이스에서 가져온 행 데이터 리스트
            table_name: 테이블명 (옵션)

        Returns:
            변환된 행 데이터 리스트
        """
        if not rows_data:
            return rows_data

        # 첫 번째 행으로 스키마 정보를 가져오고 모든 행에 적용
        converted_rows = []
        for row in rows_data:
            converted_row = await self._convert_row_data_with_schema(row, table_name)
            converted_rows.append(converted_row)

        return converted_rows

    def set_sql_loader_dir(self, sql_dir: Union[str, Path]) -> None:
        """
        SQL 로더 디렉토리를 설정합니다.

        Args:
            sql_dir: SQL 파일들이 있는 디렉토리 경로
        """
        from .sql_loader import SqlLoader

        self.sql_loader = SqlLoader(sql_dir)

    def set_sql_loader(self, sql_loader: "SqlLoader") -> None:
        """
        SQL 로더를 직접 설정합니다.

        Args:
            sql_loader: SqlLoader 인스턴스
        """
        self.sql_loader = sql_loader

    def load_sql(self, filename: str, name: Optional[str] = None) -> str:
        """
        SQL 파일에서 SQL 문을 로드합니다.

        Args:
            filename: SQL 파일명
            name: 로드할 SQL의 이름 (옵션)

        Returns:
            로드된 SQL 문

        Raises:
            ValueError: SQL 로더가 설정되지 않은 경우
        """
        if self.sql_loader is None:
            raise ValueError(
                "SQL 로더가 설정되지 않았습니다. set_sql_loader_dir() 또는 set_sql_loader()를 호출하세요."
            )

        return self.sql_loader.load_sql(filename, name)

    def _parse_dsn(self) -> tuple[str, str]:
        """
        DSN을 파싱하여 데이터베이스 타입과 연결 정보를 반환합니다.

        Returns:
            (db_type, connection_info) 튜플

        Raises:
            ValueError: 지원하지 않는 DSN 형식인 경우
        """
        if not self.dsn:
            raise ValueError("DSN이 설정되지 않았습니다.")

        parsed = urlparse(self.dsn)
        db_type = parsed.scheme

        if db_type == "sqlite":
            # SQLite: sqlite:///path/to/db.sqlite
            db_path = parsed.path
            if db_path.startswith("///"):
                db_path = db_path[3:]  # file:///path -> /path
            elif db_path.startswith("/"):
                db_path = db_path[1:]  # /path -> path (상대 경로)
            return db_type, db_path
        else:
            raise ValueError(f"지원하지 않는 데이터베이스 타입: {db_type}")

    async def connect(self) -> None:
        """
        데이터베이스에 연결합니다.
        DSN을 파싱하여 적절한 드라이버로 연결합니다.
        """
        if self.dsn is None:
            raise ValueError("DSN이 설정되지 않았습니다.")

        db_type, connection_info = self._parse_dsn()
        self._db_type = db_type  # 데이터베이스 타입 저장

        logger.info(f"데이터베이스 연결 중: {db_type}")

        if db_type == "sqlite":
            await self._connect_sqlite(connection_info)
        else:
            raise NotImplementedError(f"데이터베이스 타입 '{db_type}'는 아직 구현되지 않았습니다.")

    async def _connect_sqlite(self, db_path: str) -> None:
        """
        SQLite 데이터베이스에 연결합니다.

        Args:
            db_path: SQLite 데이터베이스 파일 경로
        """
        try:
            import aiosqlite
        except ImportError:
            raise ImportError(
                "aiosqlite가 설치되지 않았습니다. 'pip install aiosqlite' 또는 'pip install pybatis[sqlite]'를 실행하세요."
            )

        # aiosqlite 연결 생성
        self._connection = await aiosqlite.connect(db_path)
        # Row 팩토리 설정으로 딕셔너리 형태로 결과 반환
        self._connection.row_factory = aiosqlite.Row

        logger.info(f"SQLite 데이터베이스에 연결되었습니다: {db_path}")

    async def fetch_val(self, sql: str, params: Optional[Dict[str, Any]] = None) -> Any:
        """
        단일 스칼라 값을 반환합니다.

        Args:
            sql: 실행할 SQL 문
            params: SQL 파라미터

        Returns:
            단일 스칼라 값 (예: COUNT, MAX, MIN 등의 결과)
        """
        if self._connection is None:
            raise RuntimeError("데이터베이스에 연결되지 않았습니다.")

        async def executor():
            logger.debug(f"SQL 실행 (fetch_val): {sql}, params: {params}")

            # aiosqlite 연결인 경우 (aiosqlite.Connection 타입 체크)
            if hasattr(self._connection, 'execute') and hasattr(self._connection, 'row_factory'):
                async with self._connection.execute(sql, params or {}) as cursor:
                    row = await cursor.fetchone()
                    return row[0] if row else None
            else:
                # 테스트용 MockConnection
                return await self._connection.fetchval(sql, params)

        return await self._execute_with_monitoring("fetch_val", sql, params, executor)

    async def fetch_one(
        self, sql: str, params: Optional[Dict[str, Any]] = None
    ) -> Optional[Dict[str, Any]]:
        """
        하나의 레코드를 반환합니다.

        Args:
            sql: 실행할 SQL 문
            params: SQL 파라미터

        Returns:
            레코드 딕셔너리 또는 None
        """
        if self._connection is None:
            raise RuntimeError("데이터베이스에 연결되지 않았습니다.")

        async def executor():
            logger.debug(f"SQL 실행 (fetch_one): {sql}, params: {params}")

            # SQL에서 테이블명 추출
            table_name = await self._extract_table_name_from_sql(sql)

            # aiosqlite 연결인 경우
            if hasattr(self._connection, 'execute') and hasattr(self._connection, 'row_factory'):
                async with self._connection.execute(sql, params or {}) as cursor:
                    row = await cursor.fetchone()
                    if row is None:
                        return None
                    row_data = dict(row)
                    return await self._convert_row_data_with_schema(row_data, table_name)
            else:
                # 테스트용 MockConnection
                row = await self._connection.fetchrow(sql, params)
                if row is None:
                    return None
                row_data = dict(row)
                return await self._convert_row_data_with_schema(row_data, table_name)

        return await self._execute_with_monitoring("fetch_one", sql, params, executor)

    async def fetch_all(
        self, sql: str, params: Optional[Dict[str, Any]] = None
    ) -> List[Dict[str, Any]]:
        """
        모든 레코드를 반환합니다.

        Args:
            sql: 실행할 SQL 문
            params: SQL 파라미터

        Returns:
            레코드 딕셔너리들의 리스트
        """
        if self._connection is None:
            raise RuntimeError("데이터베이스에 연결되지 않았습니다.")

        async def executor():
            logger.debug(f"SQL 실행 (fetch_all): {sql}, params: {params}")

            # SQL에서 테이블명 추출
            table_name = await self._extract_table_name_from_sql(sql)

            # aiosqlite 연결인 경우
            if hasattr(self._connection, 'execute') and hasattr(self._connection, 'row_factory'):
                async with self._connection.execute(sql, params or {}) as cursor:
                    rows = await cursor.fetchall()
                    rows_data = [dict(row) for row in rows]
                    return await self._convert_rows_data_with_schema(rows_data, table_name)
            else:
                # 테스트용 MockConnection
                rows = await self._connection.fetch(sql, params)
                rows_data = [dict(row) for row in rows]
                return await self._convert_rows_data_with_schema(rows_data, table_name)

        return await self._execute_with_monitoring("fetch_all", sql, params, executor)

    async def execute(self, sql: str, params: Optional[Dict[str, Any]] = None) -> Any:
        """
        SQL 문을 실행합니다 (INSERT, UPDATE, DELETE).

        Args:
            sql: 실행할 SQL 문
            params: SQL 파라미터

        Returns:
            영향받은 행의 수 또는 기타 실행 결과
        """
        if self._connection is None:
            raise RuntimeError("데이터베이스에 연결되지 않았습니다.")

        async def executor():
            logger.debug(f"SQL 실행 (execute): {sql}, params: {params}")

            # aiosqlite 연결인 경우
            if hasattr(self._connection, 'execute') and hasattr(self._connection, 'row_factory'):
                async with self._connection.execute(sql, params or {}) as cursor:
                    # SQLite는 lastrowid (INSERT의 경우) 또는 rowcount (UPDATE/DELETE의 경우) 반환
                    return cursor.lastrowid or cursor.rowcount
            else:
                # 테스트용 MockConnection
                return await self._connection.execute(sql, params)

        return await self._execute_with_monitoring("execute", sql, params, executor)

    async def close(self) -> None:
        """
        데이터베이스 연결을 닫습니다.
        """
        if self._connection:
            if hasattr(self._connection, 'close') and hasattr(self._connection, 'row_factory'):
                # aiosqlite 연결
                await self._connection.close()
            else:
                # 테스트용 MockConnection
                await self._connection.close()
            self._connection = None
            logger.info("데이터베이스 연결이 닫혔습니다.")

    @asynccontextmanager
    async def transaction(self) -> AsyncGenerator["PyBatis", None]:
        """
        트랜잭션 컨텍스트 매니저를 제공합니다.

        트랜잭션 내에서 예외가 발생하면 자동으로 롤백되고,
        정상적으로 완료되면 커밋됩니다.

        Yields:
            트랜잭션이 활성화된 PyBatis 인스턴스

        Example:
            ```python
            async with pybatis.transaction() as tx:
                await tx.execute("INSERT INTO users (name) VALUES (?)", {"name": "John"})
                await tx.execute("INSERT INTO profiles (user_id) VALUES (?)", {"user_id": 1})
            ```
        """
        if self._connection is None:
            raise RuntimeError("데이터베이스에 연결되지 않았습니다.")

        # aiosqlite 연결인 경우
        if hasattr(self._connection, 'execute') and hasattr(self._connection, 'row_factory'):
            # SQLite는 기본적으로 autocommit이 비활성화되어 있음
            # 트랜잭션 시작
            if self._query_logging_enabled:
                logger.log(self._query_logging_level, "트랜잭션 시작")

            await self._connection.execute("BEGIN")
            try:
                yield self
                # 정상 완료 시 커밋
                await self._connection.commit()
                if self._query_logging_enabled:
                    logger.log(self._query_logging_level, "트랜잭션 커밋")
            except Exception:
                # 예외 발생 시 롤백
                await self._connection.rollback()
                if self._query_logging_enabled:
                    logger.log(self._query_logging_level, "트랜잭션 롤백")
                raise
        else:
            # 테스트용 MockConnection - 단순히 self 반환
            if self._query_logging_enabled:
                logger.log(self._query_logging_level, "트랜잭션 시작 (MockConnection)")
            yield self
            if self._query_logging_enabled:
                logger.log(self._query_logging_level, "트랜잭션 완료 (MockConnection)")

    async def __aenter__(self) -> "PyBatis":
        """비동기 컨텍스트 매니저 진입"""
        if self.dsn and self._connection is None:
            await self.connect()
        return self

    async def __aexit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
        """비동기 컨텍스트 매니저 종료"""
        await self.close()



"""
SQL 파일 로딩 기능 테스트

.sql 파일에서 SQL 문을 로드하는 기능을 테스트합니다.
"""

import tempfile
from pathlib import Path

import pytest

from pybatis.sql_loader import SqlLoader


class TestSqlLoader:
    """SQL 로더 클래스의 테스트"""

    @pytest.fixture
    def temp_sql_dir(self):
        """임시 SQL 디렉토리 픽스처"""
        with tempfile.TemporaryDirectory() as temp_dir:
            sql_dir = Path(temp_dir)

            # 샘플 SQL 파일들 생성
            (sql_dir / "users.sql").write_text(
                """
-- 사용자 조회 SQL
SELECT id, name, email, is_active
FROM users
WHERE id = :user_id
""".strip()
            )

            (sql_dir / "user_operations.sql").write_text(
                """
-- name=get_user_by_email
SELECT id, name, email, is_active
FROM users
WHERE email = :email

-- name=count_active_users
SELECT COUNT(*)
FROM users
WHERE is_active = :active

-- name=insert_user
INSERT INTO users (name, email, is_active)
VALUES (:name, :email, :is_active)
""".strip()
            )

            yield sql_dir

    def test_create_sql_loader(self, temp_sql_dir):
        """SQL 로더 인스턴스 생성 테스트"""
        loader = SqlLoader(sql_dir=temp_sql_dir)
        assert loader.sql_dir == temp_sql_dir

    def test_load_sql_from_file(self, temp_sql_dir):
        """단일 SQL 파일 로드 테스트"""
        loader = SqlLoader(sql_dir=temp_sql_dir)

        sql = loader.load_sql("users.sql")

        assert "SELECT id, name, email, is_active" in sql
        assert "FROM users" in sql
        assert "WHERE id = :user_id" in sql

    def test_load_sql_with_name(self, temp_sql_dir):
        """이름으로 SQL 로드 테스트"""
        loader = SqlLoader(sql_dir=temp_sql_dir)

        sql = loader.load_sql("user_operations.sql", name="get_user_by_email")

        assert "SELECT id, name, email, is_active" in sql
        assert "WHERE email = :email" in sql
        assert "COUNT(*)" not in sql  # 다른 SQL은 포함되지 않음

    def test_load_sql_count_query(self, temp_sql_dir):
        """카운트 쿼리 로드 테스트"""
        loader = SqlLoader(sql_dir=temp_sql_dir)

        sql = loader.load_sql("user_operations.sql", name="count_active_users")

        assert "SELECT COUNT(*)" in sql
        assert "WHERE is_active = :active" in sql

    def test_load_sql_insert_query(self, temp_sql_dir):
        """인서트 쿼리 로드 테스트"""
        loader = SqlLoader(sql_dir=temp_sql_dir)

        sql = loader.load_sql("user_operations.sql", name="insert_user")

        assert "INSERT INTO users" in sql
        assert "VALUES (:name, :email, :is_active)" in sql

    def test_load_sql_file_not_found(self, temp_sql_dir):
        """존재하지 않는 파일 로드 시 예외 테스트"""
        loader = SqlLoader(sql_dir=temp_sql_dir)

        with pytest.raises(FileNotFoundError):
            loader.load_sql("nonexistent.sql")

    def test_load_sql_name_not_found(self, temp_sql_dir):
        """존재하지 않는 이름 로드 시 예외 테스트"""
        loader = SqlLoader(sql_dir=temp_sql_dir)

        with pytest.raises(ValueError, match="SQL with name 'nonexistent' not found"):
            loader.load_sql("user_operations.sql", name="nonexistent")

    def test_load_sql_without_name_from_multi_sql_file(self, temp_sql_dir):
        """이름 없이 다중 SQL 파일 로드 시 전체 내용 반환 테스트"""
        loader = SqlLoader(sql_dir=temp_sql_dir)

        sql = loader.load_sql("user_operations.sql")

        # 전체 파일 내용이 반환되어야 함
        assert "get_user_by_email" in sql
        assert "count_active_users" in sql
        assert "insert_user" in sql


class TestSqlLoaderIntegration:
    """SQL 로더와 PyBatis 통합 테스트"""

    @pytest.fixture
    def temp_sql_dir(self):
        """임시 SQL 디렉토리 픽스처"""
        with tempfile.TemporaryDirectory() as temp_dir:
            sql_dir = Path(temp_dir)

            (sql_dir / "users.sql").write_text(
                """
SELECT id, name, email, is_active
FROM users
WHERE id = :user_id
""".strip()
            )

            yield sql_dir

    def test_pybatis_with_sql_loader(self, temp_sql_dir):
        """PyBatis와 SQL 로더 통합 테스트"""
        from pybatis import PyBatis
        from tests.test_pybatis import MockConnection

        # PyBatis에 SQL 로더 설정
        db = PyBatis()
        db.set_sql_loader(SqlLoader(sql_dir=temp_sql_dir))
        db._connection = MockConnection()  # 테스트용 연결 주입

        # SQL 파일에서 로드한 SQL 실행 테스트는 실제 구현 후 추가
        assert db.sql_loader is not None
        assert db.sql_loader.sql_dir == temp_sql_dir

    def test_pybatis_init_with_sql_dir(self, temp_sql_dir):
        """PyBatis 초기화 시 SQL 디렉토리 설정 테스트"""
        from pybatis import PyBatis

        db = PyBatis(sql_dir=temp_sql_dir)

        assert db.sql_loader is not None
        assert db.sql_loader.sql_dir == temp_sql_dir

    def test_load_sql_from_pybatis(self, temp_sql_dir):
        """PyBatis에서 SQL 로드 테스트"""
        from pybatis import PyBatis

        db = PyBatis(sql_dir=temp_sql_dir)

        sql = db.load_sql("users.sql")

        assert "SELECT id, name, email, is_active" in sql
        assert "FROM users" in sql
        assert "WHERE id = :user_id" in sql

    def test_load_sql_without_loader_raises_error(self):
        """SQL 로더 없이 SQL 로드 시 오류 테스트"""
        from pybatis import PyBatis

        db = PyBatis()

        with pytest.raises(ValueError, match="SQL 로더가 설정되지 않았습니다"):
            db.load_sql("users.sql")


class TestSqlLoaderUtilityMethods:
    """SQL 로더 유틸리티 메서드 테스트"""

    @pytest.fixture
    def temp_sql_dir(self):
        """임시 SQL 디렉토리 픽스처"""
        with tempfile.TemporaryDirectory() as temp_dir:
            sql_dir = Path(temp_dir)

            (sql_dir / "users.sql").write_text("SELECT * FROM users")
            (sql_dir / "posts.sql").write_text("SELECT * FROM posts")
            (sql_dir / "comments.sql").write_text(
                """
-- name=get_comment
SELECT * FROM comments WHERE id = :id

-- name=list_comments
SELECT * FROM comments WHERE post_id = :post_id
""".strip()
            )

            yield sql_dir

    def test_list_sql_files(self, temp_sql_dir):
        """SQL 파일 목록 조회 테스트"""
        loader = SqlLoader(sql_dir=temp_sql_dir)

        files = loader.list_sql_files()

        assert "users.sql" in files
        assert "posts.sql" in files
        assert "comments.sql" in files
        assert len(files) == 3

    def test_list_named_sqls(self, temp_sql_dir):
        """이름이 지정된 SQL 목록 조회 테스트"""
        loader = SqlLoader(sql_dir=temp_sql_dir)

        names = loader.list_named_sqls("comments.sql")

        assert "get_comment" in names
        assert "list_comments" in names
        assert len(names) == 2

    def test_list_named_sqls_empty_file(self, temp_sql_dir):
        """이름이 없는 SQL 파일에서 빈 목록 반환 테스트"""
        loader = SqlLoader(sql_dir=temp_sql_dir)

        names = loader.list_named_sqls("users.sql")

        assert names == []



"""
SQLite 통합 테스트

aiosqlite를 사용한 실제 데이터베이스 연결 및 쿼리 실행을 테스트합니다.
"""

import tempfile
from pathlib import Path

import pytest

from pybatis import PyBatis
from tests.fixtures import User


class TestSQLiteIntegration:
    """SQLite 데이터베이스 통합 테스트"""

    @pytest.fixture
    async def temp_db(self):
        """임시 SQLite 데이터베이스 픽스처"""
        with tempfile.NamedTemporaryFile(suffix=".db", delete=False) as temp_file:
            db_path = temp_file.name

        # DSN 생성
        dsn = f"sqlite:///{db_path}"

        try:
            # 테이블 생성을 위한 초기 연결
            db = PyBatis(dsn=dsn)
            await db.connect()

            # 테스트용 테이블 생성
            await db.execute("""
                CREATE TABLE users (
                    id INTEGER PRIMARY KEY AUTOINCREMENT,
                    name TEXT NOT NULL,
                    email TEXT UNIQUE NOT NULL,
                    is_active BOOLEAN DEFAULT TRUE
                )
            """)

            # 테스트 데이터 삽입
            await db.execute(
                "INSERT INTO users (name, email, is_active) VALUES (:name, :email, :is_active)",
                params={"name": "테스트사용자", "email": "test@example.com", "is_active": True},
            )
            await db.execute(
                "INSERT INTO users (name, email, is_active) VALUES (:name, :email, :is_active)",
                params={"name": "비활성사용자", "email": "inactive@example.com", "is_active": False},
            )

            yield db

        finally:
            await db.close()
            # 임시 파일 정리
            Path(db_path).unlink(missing_ok=True)

    @pytest.mark.asyncio
    async def test_sqlite_connection(self):
        """SQLite 연결 테스트"""
        with tempfile.NamedTemporaryFile(suffix=".db") as temp_file:
            dsn = f"sqlite:///{temp_file.name}"

            db = PyBatis(dsn=dsn)
            await db.connect()

            assert db._connection is not None

            await db.close()

    @pytest.mark.asyncio
    async def test_fetch_val_with_sqlite(self, temp_db):
        """SQLite에서 fetch_val 테스트"""
        count = await temp_db.fetch_val("SELECT COUNT(*) FROM users")

        assert count == 2

    @pytest.mark.asyncio
    async def test_fetch_val_with_params(self, temp_db):
        """SQLite에서 파라미터를 사용한 fetch_val 테스트"""
        count = await temp_db.fetch_val(
            "SELECT COUNT(*) FROM users WHERE is_active = :active", params={"active": True}
        )

        assert count == 1

    @pytest.mark.asyncio
    async def test_fetch_one_with_sqlite(self, temp_db):
        """SQLite에서 fetch_one 테스트"""
        row = await temp_db.fetch_one(
            "SELECT id, name, email, is_active FROM users WHERE name = :name",
            params={"name": "테스트사용자"},
        )

        assert row is not None
        assert row["name"] == "테스트사용자"
        assert row["email"] == "test@example.com"
        assert row["is_active"] == 1  # SQLite에서 boolean은 integer로 저장됨

    @pytest.mark.asyncio
    async def test_fetch_all_with_sqlite(self, temp_db):
        """SQLite에서 fetch_all 테스트"""
        rows = await temp_db.fetch_all("SELECT id, name, email, is_active FROM users")

        assert len(rows) == 2
        assert rows[0]["name"] == "테스트사용자"
        assert rows[1]["name"] == "비활성사용자"

    @pytest.mark.asyncio
    async def test_execute_insert(self, temp_db):
        """SQLite에서 INSERT 실행 테스트"""
        result = await temp_db.execute(
            "INSERT INTO users (name, email, is_active) VALUES (:name, :email, :is_active)",
            params={"name": "새사용자", "email": "new@example.com", "is_active": True},
        )

        # SQLite에서는 영향받은 행 수 또는 마지막 insert row id 반환
        assert result is not None

        # 삽입된 데이터 확인
        count = await temp_db.fetch_val("SELECT COUNT(*) FROM users")
        assert count == 3

    @pytest.mark.asyncio
    async def test_execute_update(self, temp_db):
        """SQLite에서 UPDATE 실행 테스트"""
        result = await temp_db.execute(
            "UPDATE users SET is_active = :is_active WHERE name = :name",
            params={"is_active": False, "name": "테스트사용자"},
        )

        assert result is not None

        # 업데이트된 데이터 확인
        row = await temp_db.fetch_one(
            "SELECT is_active FROM users WHERE name = :name", params={"name": "테스트사용자"}
        )
        assert row["is_active"] == 0  # False는 0으로 저장됨

    @pytest.mark.asyncio
    async def test_execute_delete(self, temp_db):
        """SQLite에서 DELETE 실행 테스트"""
        result = await temp_db.execute(
            "DELETE FROM users WHERE name = :name", params={"name": "비활성사용자"}
        )

        assert result is not None

        # 삭제 확인
        count = await temp_db.fetch_val("SELECT COUNT(*) FROM users")
        assert count == 1

    @pytest.mark.asyncio
    async def test_context_manager(self):
        """SQLite 컨텍스트 매니저 테스트"""
        with tempfile.NamedTemporaryFile(suffix=".db") as temp_file:
            dsn = f"sqlite:///{temp_file.name}"

            async with PyBatis(dsn=dsn) as db:
                # 테이블 생성
                await db.execute("""
                    CREATE TABLE test_table (
                        id INTEGER PRIMARY KEY,
                        name TEXT
                    )
                """)

                # 데이터 삽입
                await db.execute(
                    "INSERT INTO test_table (name) VALUES (:name)", params={"name": "테스트"}
                )

                # 데이터 조회
                count = await db.fetch_val("SELECT COUNT(*) FROM test_table")
                assert count == 1

            # 컨텍스트 매니저 종료 후 연결이 닫혔는지 확인
            # (실제 테스트에서는 연결이 이미 닫혔으므로 에러가 발생해야 함)


class TestSQLitePydanticIntegration:
    """SQLite와 Pydantic 모델 통합 테스트"""

    @pytest.fixture
    async def temp_db_with_users(self):
        """사용자 데이터가 있는 임시 SQLite 데이터베이스 픽스처"""
        with tempfile.NamedTemporaryFile(suffix=".db", delete=False) as temp_file:
            db_path = temp_file.name

        dsn = f"sqlite:///{db_path}"

        try:
            db = PyBatis(dsn=dsn)
            await db.connect()

            # 테스트용 테이블 생성
            await db.execute("""
                CREATE TABLE users (
                    id INTEGER PRIMARY KEY AUTOINCREMENT,
                    name TEXT NOT NULL,
                    email TEXT UNIQUE NOT NULL,
                    is_active BOOLEAN DEFAULT TRUE
                )
            """)

            # 테스트 데이터 삽입
            await db.execute(
                "INSERT INTO users (name, email, is_active) VALUES (:name, :email, :is_active)",
                params={"name": "김철수", "email": "kim@example.com", "is_active": True},
            )
            await db.execute(
                "INSERT INTO users (name, email, is_active) VALUES (:name, :email, :is_active)",
                params={"name": "이영희", "email": "lee@example.com", "is_active": True},
            )
            await db.execute(
                "INSERT INTO users (name, email, is_active) VALUES (:name, :email, :is_active)",
                params={"name": "박민수", "email": "park@example.com", "is_active": False},
            )

            yield db

        finally:
            await db.close()
            Path(db_path).unlink(missing_ok=True)

    @pytest.mark.asyncio
    async def test_fetch_one_to_pydantic_model(self, temp_db_with_users):
        """SQLite에서 조회한 데이터를 Pydantic 모델로 변환 테스트"""
        row = await temp_db_with_users.fetch_one(
            "SELECT id, name, email, is_active FROM users WHERE name = :name",
            params={"name": "김철수"},
        )

        assert row is not None

        # 딕셔너리를 Pydantic 모델로 변환
        # SQLite에서 boolean은 integer로 저장되므로 변환 필요
        user_data = dict(row)
        user_data["is_active"] = bool(user_data["is_active"])  # 1 -> True, 0 -> False
        user = User(**user_data)

        assert isinstance(user, User)
        assert user.name == "김철수"
        assert user.email == "kim@example.com"
        assert user.is_active is True
        assert user.id == 1

    @pytest.mark.asyncio
    async def test_fetch_all_to_pydantic_models(self, temp_db_with_users):
        """SQLite에서 조회한 모든 데이터를 Pydantic 모델 리스트로 변환 테스트"""
        rows = await temp_db_with_users.fetch_all(
            "SELECT id, name, email, is_active FROM users WHERE is_active = :active",
            params={"active": True},
        )

        assert len(rows) == 2

        # 딕셔너리 리스트를 Pydantic 모델 리스트로 변환
        users = []
        for row in rows:
            user_data = dict(row)
            user_data["is_active"] = bool(user_data["is_active"])
            users.append(User(**user_data))

        assert len(users) == 2
        assert all(isinstance(user, User) for user in users)
        assert users[0].name == "김철수"
        assert users[1].name == "이영희"
        assert all(user.is_active for user in users)

    @pytest.mark.asyncio
    async def test_repository_pattern_with_sqlite(self, temp_db_with_users):
        """SQLite를 사용한 Repository 패턴과 Pydantic 모델 통합 테스트"""
        from tests.fixtures import UserRepository

        repo = UserRepository(temp_db_with_users)

        # 단일 사용자 조회
        user = await repo.get_user_by_id(1)
        assert isinstance(user, User)
        assert user.name == "김철수"
        assert user.email == "kim@example.com"
        assert user.is_active is True

        # 존재하지 않는 사용자 조회
        user = await repo.get_user_by_id(999)
        assert user is None

        # 활성 사용자 목록 조회
        active_users = await repo.get_users_by_activity(True)
        assert len(active_users) == 2
        assert all(isinstance(user, User) for user in active_users)
        assert all(user.is_active for user in active_users)

        # 비활성 사용자 목록 조회
        inactive_users = await repo.get_users_by_activity(False)
        assert len(inactive_users) == 1
        assert isinstance(inactive_users[0], User)
        assert inactive_users[0].name == "박민수"
        assert inactive_users[0].is_active is False

        # 활성 사용자 수 조회
        count = await repo.count_active(True)
        assert count == 2


class TestSQLiteWithSqlLoader:
    """SQLite와 SQL 로더 통합 테스트"""

    @pytest.fixture
    async def temp_db_with_sql(self):
        """SQL 파일과 함께 SQLite 데이터베이스 픽스처"""
        with tempfile.NamedTemporaryFile(suffix=".db", delete=False) as temp_file:
            db_path = temp_file.name

        with tempfile.TemporaryDirectory() as temp_dir:
            sql_dir = Path(temp_dir)

            # SQL 파일 생성
            (sql_dir / "users.sql").write_text("""
-- name=get_user_by_id
SELECT id, name, email, is_active
FROM users
WHERE id = :id

-- name=count_active_users
SELECT COUNT(*)
FROM users
WHERE is_active = :active

-- name=insert_user
INSERT INTO users (name, email, is_active)
VALUES (:name, :email, :is_active)
""".strip())

            # DSN 생성
            dsn = f"sqlite:///{db_path}"

            try:
                # PyBatis 인스턴스 생성 (SQL 디렉토리 포함)
                db = PyBatis(dsn=dsn, sql_dir=sql_dir)
                await db.connect()

                # 테스트용 테이블 생성
                await db.execute("""
                    CREATE TABLE users (
                        id INTEGER PRIMARY KEY AUTOINCREMENT,
                        name TEXT NOT NULL,
                        email TEXT UNIQUE NOT NULL,
                        is_active BOOLEAN DEFAULT TRUE
                    )
                """)

                yield db

            finally:
                await db.close()
                Path(db_path).unlink(missing_ok=True)

    @pytest.mark.asyncio
    async def test_fetch_with_loaded_sql(self, temp_db_with_sql):
        """SQL 파일에서 로드한 SQL로 데이터 조회 테스트"""
        # 먼저 데이터 삽입
        insert_sql = temp_db_with_sql.load_sql("users.sql", name="insert_user")
        await temp_db_with_sql.execute(insert_sql, params={"name": "테스트", "email": "test@example.com", "is_active": True})

        # 삽입된 데이터 조회
        select_sql = temp_db_with_sql.load_sql("users.sql", name="get_user_by_id")
        row = await temp_db_with_sql.fetch_one(select_sql, params={"id": 1})

        assert row is not None
        assert row["name"] == "테스트"
        assert row["email"] == "test@example.com"

    @pytest.mark.asyncio
    async def test_count_with_loaded_sql(self, temp_db_with_sql):
        """SQL 파일에서 로드한 SQL로 카운트 테스트"""
        # 데이터 삽입
        insert_sql = temp_db_with_sql.load_sql("users.sql", name="insert_user")
        await temp_db_with_sql.execute(insert_sql, params={"name": "활성사용자", "email": "active@example.com", "is_active": True})
        await temp_db_with_sql.execute(insert_sql, params={"name": "비활성사용자", "email": "inactive@example.com", "is_active": False})

        # 활성 사용자 수 조회
        count_sql = temp_db_with_sql.load_sql("users.sql", name="count_active_users")
        count = await temp_db_with_sql.fetch_val(count_sql, params={"active": True})

        assert count == 1


class TestSQLiteRepository:
    """SQLite를 사용한 Repository 패턴 테스트"""

    @pytest.fixture
    async def user_repository_with_sqlite(self):
        """SQLite를 사용하는 UserRepository 픽스처"""
        with tempfile.NamedTemporaryFile(suffix=".db", delete=False) as temp_file:
            db_path = temp_file.name

        dsn = f"sqlite:///{db_path}"

        try:
            from tests.fixtures import UserRepository

            db = PyBatis(dsn=dsn)
            await db.connect()

            # 테이블 생성
            await db.execute("""
                CREATE TABLE users (
                    id INTEGER PRIMARY KEY AUTOINCREMENT,
                    name TEXT NOT NULL,
                    email TEXT UNIQUE NOT NULL,
                    is_active BOOLEAN DEFAULT TRUE
                )
            """)

            repository = UserRepository(db)
            yield repository

        finally:
            await db.close()
            Path(db_path).unlink(missing_ok=True)

    @pytest.mark.asyncio
    async def test_repository_with_real_database(self, user_repository_with_sqlite):
        """실제 데이터베이스를 사용한 Repository 테스트"""
        repo = user_repository_with_sqlite

        # 먼저 사용자 데이터 삽입 (SQL을 직접 실행)
        await repo.db.execute(
            "INSERT INTO users (name, email, is_active) VALUES (:name, :email, :is_active)",
            params={"name": "실제사용자", "email": "real@example.com", "is_active": True},
        )

        # Repository 메서드로 데이터 조회
        count = await repo.count_active(active=True)
        assert count == 1


class TestSQLiteSchemaBasedBooleanConversion:
    """SQLite 스키마 기반 boolean 자동 변환 테스트"""

    @pytest.fixture
    async def temp_db_with_boolean_schema(self):
        """BOOLEAN 타입 컬럼이 있는 임시 SQLite 데이터베이스 픽스처"""
        with tempfile.NamedTemporaryFile(suffix=".db", delete=False) as temp_file:
            db_path = temp_file.name

        dsn = f"sqlite:///{db_path}"

        try:
            db = PyBatis(dsn=dsn)
            await db.connect()

            # BOOLEAN 타입으로 명시적으로 정의된 테이블 생성
            await db.execute("""
                CREATE TABLE test_users (
                    id INTEGER PRIMARY KEY AUTOINCREMENT,
                    name TEXT NOT NULL,
                    email TEXT UNIQUE NOT NULL,
                    is_active BOOLEAN DEFAULT TRUE,
                    is_verified BOOLEAN DEFAULT FALSE,
                    has_premium BOOLEAN DEFAULT FALSE
                )
            """)

            # 테스트 데이터 삽입
            await db.execute(
                "INSERT INTO test_users (name, email, is_active, is_verified, has_premium) VALUES (:name, :email, :is_active, :is_verified, :has_premium)",
                params={"name": "테스트사용자1", "email": "test1@example.com", "is_active": True, "is_verified": False, "has_premium": True},
            )
            await db.execute(
                "INSERT INTO test_users (name, email, is_active, is_verified, has_premium) VALUES (:name, :email, :is_active, :is_verified, :has_premium)",
                params={"name": "테스트사용자2", "email": "test2@example.com", "is_active": False, "is_verified": True, "has_premium": False},
            )

            yield db

        finally:
            await db.close()
            Path(db_path).unlink(missing_ok=True)

    @pytest.mark.asyncio
    async def test_schema_based_boolean_conversion_fetch_one(self, temp_db_with_boolean_schema):
        """스키마 기반 boolean 변환 - fetch_one 테스트"""
        row = await temp_db_with_boolean_schema.fetch_one(
            "SELECT id, name, email, is_active, is_verified, has_premium FROM test_users WHERE name = :name",
            params={"name": "테스트사용자1"},
        )

        assert row is not None
        assert row["name"] == "테스트사용자1"
        assert row["email"] == "test1@example.com"

        # 스키마 기반 boolean 자동 변환 확인
        assert row["is_active"] is True  # True -> True
        assert row["is_verified"] is False  # False -> False
        assert row["has_premium"] is True  # True -> True

        # 타입 확인
        assert isinstance(row["is_active"], bool)
        assert isinstance(row["is_verified"], bool)
        assert isinstance(row["has_premium"], bool)

    @pytest.mark.asyncio
    async def test_schema_based_boolean_conversion_fetch_all(self, temp_db_with_boolean_schema):
        """스키마 기반 boolean 변환 - fetch_all 테스트"""
        rows = await temp_db_with_boolean_schema.fetch_all(
            "SELECT id, name, email, is_active, is_verified, has_premium FROM test_users ORDER BY id"
        )

        assert len(rows) == 2

        # 첫 번째 사용자
        user1 = rows[0]
        assert user1["name"] == "테스트사용자1"
        assert user1["is_active"] is True
        assert user1["is_verified"] is False
        assert user1["has_premium"] is True
        assert all(isinstance(user1[col], bool) for col in ["is_active", "is_verified", "has_premium"])

        # 두 번째 사용자
        user2 = rows[1]
        assert user2["name"] == "테스트사용자2"
        assert user2["is_active"] is False
        assert user2["is_verified"] is True
        assert user2["has_premium"] is False
        assert all(isinstance(user2[col], bool) for col in ["is_active", "is_verified", "has_premium"])

    @pytest.mark.asyncio
    async def test_schema_info_caching(self, temp_db_with_boolean_schema):
        """스키마 정보 캐싱 테스트"""
        db = temp_db_with_boolean_schema

        # 첫 번째 쿼리 - 스키마 정보 로드
        await db.fetch_one("SELECT * FROM test_users WHERE id = 1")

        # 스키마 캐시 확인
        assert "test_users" in db._table_schemas
        schema = db._table_schemas["test_users"]
        assert schema["is_active"] == "BOOLEAN"
        assert schema["is_verified"] == "BOOLEAN"
        assert schema["has_premium"] == "BOOLEAN"
        assert schema["name"] == "TEXT"
        assert schema["email"] == "TEXT"

    @pytest.mark.asyncio
    async def test_non_boolean_columns_unchanged(self, temp_db_with_boolean_schema):
        """boolean이 아닌 컬럼은 변환되지 않는지 테스트"""
        row = await temp_db_with_boolean_schema.fetch_one(
            "SELECT id, name, email FROM test_users WHERE id = 1"
        )

        assert row is not None
        assert isinstance(row["id"], int)
        assert isinstance(row["name"], str)
        assert isinstance(row["email"], str)

    @pytest.mark.asyncio
    async def test_table_name_extraction(self, temp_db_with_boolean_schema):
        """SQL에서 테이블명 추출 테스트"""
        db = temp_db_with_boolean_schema

        # 다양한 SQL 패턴에서 테이블명 추출 테스트
        test_cases = [
            ("SELECT * FROM test_users", "test_users"),
            ("SELECT id, name FROM test_users WHERE id = 1", "test_users"),
            ("UPDATE test_users SET name = 'test'", "test_users"),
            ("INSERT INTO test_users (name) VALUES ('test')", "test_users"),
        ]

        for sql, expected_table in test_cases:
            table_name = await db._extract_table_name_from_sql(sql)
            assert table_name == expected_table


"""FastAPI 의존성 주입 통합 테스트"""

import pytest
from unittest.mock import AsyncMock, MagicMock, patch
from fastapi import FastAPI, Depends
from fastapi.testclient import TestClient
from pydantic import BaseModel

from pybatis.fastapi import (
    get_pybatis,
    PyBatisManager,
    create_pybatis_dependency,
    transaction_context,
    _default_manager,
)
from pybatis import PyBatis


class User(BaseModel):
    id: int
    name: str
    email: str


class TestPyBatisManager:
    """PyBatisManager 클래스 테스트"""

    def test_init_with_dsn(self):
        """DSN으로 초기화 테스트"""
        manager = PyBatisManager(dsn="sqlite:///:memory:")
        assert manager.dsn == "sqlite:///:memory:"
        assert manager.pybatis is None

    def test_init_with_pybatis_instance(self):
        """PyBatis 인스턴스로 초기화 테스트"""
        pybatis = PyBatis("sqlite:///:memory:")
        manager = PyBatisManager(pybatis=pybatis)
        assert manager.pybatis is pybatis
        assert manager.dsn is None

    def test_init_without_params_raises_error(self):
        """매개변수 없이 초기화 시 에러 발생 테스트"""
        with pytest.raises(ValueError, match="dsn 또는 pybatis 중 하나는 반드시 제공되어야 합니다"):
            PyBatisManager()

    def test_init_with_both_params_raises_error(self):
        """두 매개변수 모두 제공 시 에러 발생 테스트"""
        pybatis = PyBatis("sqlite:///:memory:")
        with pytest.raises(ValueError, match="dsn과 pybatis는 동시에 제공될 수 없습니다"):
            PyBatisManager(dsn="sqlite:///:memory:", pybatis=pybatis)

    @pytest.mark.asyncio
    async def test_get_pybatis_with_dsn(self):
        """DSN으로 PyBatis 인스턴스 생성 테스트"""
        manager = PyBatisManager(dsn="sqlite:///:memory:")
        pybatis = await manager.get_pybatis()

        assert isinstance(pybatis, PyBatis)
        assert pybatis.dsn == "sqlite:///:memory:"

        # 같은 인스턴스 반환 확인
        pybatis2 = await manager.get_pybatis()
        assert pybatis is pybatis2

    @pytest.mark.asyncio
    async def test_get_pybatis_with_instance(self):
        """PyBatis 인스턴스로 초기화된 경우 테스트"""
        original_pybatis = PyBatis("sqlite:///:memory:")
        manager = PyBatisManager(pybatis=original_pybatis)

        pybatis = await manager.get_pybatis()
        assert pybatis is original_pybatis

    @pytest.mark.asyncio
    async def test_close(self):
        """리소스 정리 테스트"""
        manager = PyBatisManager(dsn="sqlite:///:memory:")
        pybatis = await manager.get_pybatis()

        # close 메서드 모킹
        pybatis.close = AsyncMock()

        await manager.close()
        pybatis.close.assert_called_once()


class TestCreatePyBatisDependency:
    """create_pybatis_dependency 함수 테스트"""

    @pytest.mark.asyncio
    async def test_dependency_function_creation(self):
        """의존성 함수 생성 테스트"""
        manager = PyBatisManager(dsn="sqlite:///:memory:")
        dependency = create_pybatis_dependency(manager)

        # 의존성 함수 호출
        pybatis = await dependency()
        assert isinstance(pybatis, PyBatis)

    @pytest.mark.asyncio
    async def test_dependency_with_custom_manager(self):
        """커스텀 매니저로 의존성 생성 테스트"""
        original_pybatis = PyBatis("sqlite:///:memory:")
        manager = PyBatisManager(pybatis=original_pybatis)
        dependency = create_pybatis_dependency(manager)

        pybatis = await dependency()
        assert pybatis is original_pybatis


class TestGetPyBatisDefault:
    """기본 get_pybatis 의존성 함수 테스트"""

    def setup_method(self):
        """각 테스트 전에 전역 상태 초기화"""
        import pybatis.fastapi
        pybatis.fastapi._default_manager = None

    @patch.dict('os.environ', {'DATABASE_URL': 'sqlite:///:memory:'})
    @pytest.mark.asyncio
    async def test_get_pybatis_with_env_var(self):
        """환경변수로 설정된 DATABASE_URL 사용 테스트"""
        pybatis = await get_pybatis()
        assert isinstance(pybatis, PyBatis)
        assert pybatis.dsn == "sqlite:///:memory:"

    @patch.dict('os.environ', {}, clear=True)
    @pytest.mark.asyncio
    async def test_get_pybatis_without_env_var_raises_error(self):
        """환경변수 없이 호출 시 에러 발생 테스트"""
        with pytest.raises(ValueError, match="DATABASE_URL 환경변수가 설정되지 않았습니다"):
            await get_pybatis()


class TestTransactionContext:
    """transaction_context 함수 테스트"""

    @pytest.mark.asyncio
    async def test_transaction_context_success(self):
        """트랜잭션 성공 케이스 테스트"""
        # PyBatis 인스턴스 생성 (실제 transaction 메서드 포함)
        pybatis = PyBatis("sqlite:///:memory:")

        # transaction 메서드를 모킹
        mock_transaction = AsyncMock()
        mock_transaction.__aenter__ = AsyncMock(return_value=pybatis)
        mock_transaction.__aexit__ = AsyncMock(return_value=None)

        # patch.object를 사용하여 transaction 메서드를 모킹
        with patch.object(pybatis, 'transaction', return_value=mock_transaction) as mock_method:
            async with transaction_context(pybatis) as tx:
                assert tx is pybatis

        # transaction 메서드가 호출되었는지 확인
        mock_method.assert_called_once()

    @pytest.mark.asyncio
    async def test_transaction_context_with_exception(self):
        """트랜잭션 중 예외 발생 테스트"""
        # PyBatis 인스턴스 생성
        pybatis = PyBatis("sqlite:///:memory:")

        # transaction 메서드를 모킹
        mock_transaction = AsyncMock()
        mock_transaction.__aenter__ = AsyncMock(return_value=pybatis)
        mock_transaction.__aexit__ = AsyncMock(return_value=None)

        # patch.object를 사용하여 transaction 메서드를 모킹
        with patch.object(pybatis, 'transaction', return_value=mock_transaction) as mock_method:
            with pytest.raises(ValueError, match="test error"):
                async with transaction_context(pybatis) as tx:
                    assert tx is pybatis
                    raise ValueError("test error")

        # transaction 메서드가 호출되었는지 확인
        mock_method.assert_called_once()


class TestFastAPIIntegration:
    """FastAPI 애플리케이션과의 통합 테스트"""

    def test_fastapi_app_with_dependency(self):
        """FastAPI 앱에서 의존성 주입 테스트"""
        app = FastAPI()

        # 테스트용 매니저 생성
        manager = PyBatisManager(dsn="sqlite:///:memory:")
        dependency = create_pybatis_dependency(manager)

        @app.get("/users/{user_id}")
        async def get_user(user_id: int, pybatis: PyBatis = Depends(dependency)):
            # 모킹된 결과 반환
            return {"id": user_id, "name": "Test User", "email": "test@example.com"}

        client = TestClient(app)
        response = client.get("/users/1")

        assert response.status_code == 200
        assert response.json() == {
            "id": 1,
            "name": "Test User",
            "email": "test@example.com"
        }

    def setup_method(self):
        """각 테스트 전에 전역 상태 초기화"""
        import pybatis.fastapi
        pybatis.fastapi._default_manager = None

    @patch.dict('os.environ', {'DATABASE_URL': 'sqlite:///:memory:'})
    def test_fastapi_app_with_default_dependency(self):
        """기본 의존성을 사용한 FastAPI 앱 테스트"""
        app = FastAPI()

        @app.get("/health")
        async def health_check(pybatis: PyBatis = Depends(get_pybatis)):
            return {"status": "ok", "database": "connected"}

        client = TestClient(app)
        response = client.get("/health")

        assert response.status_code == 200
        assert response.json() == {"status": "ok", "database": "connected"}

    def test_fastapi_app_with_transaction(self):
        """트랜잭션을 사용한 FastAPI 앱 테스트"""
        app = FastAPI()

        manager = PyBatisManager(dsn="sqlite:///:memory:")
        dependency = create_pybatis_dependency(manager)

        @app.post("/users")
        async def create_user(
            user_data: dict,
            pybatis: PyBatis = Depends(dependency)
        ):
            # 트랜잭션 컨텍스트 사용 예시 (실제 DB 작업 없이 모킹)
            # transaction_context를 모킹하여 실제 DB 연결 없이 테스트
            return {"id": 1, "created": True}

        client = TestClient(app)
        response = client.post("/users", json={"name": "New User", "email": "new@example.com"})

        assert response.status_code == 200
        assert response.json() == {"id": 1, "created": True}


class TestRepositoryWithFastAPI:
    """Repository 패턴과 FastAPI 통합 테스트"""

    def test_repository_dependency_injection(self):
        """Repository를 의존성으로 주입하는 테스트"""
        app = FastAPI()

        class UserRepository:
            def __init__(self, pybatis: PyBatis):
                self.pybatis = pybatis

            async def get_user(self, user_id: int) -> User:
                # 모킹된 결과 반환
                return User(id=user_id, name="Test User", email="test@example.com")

        manager = PyBatisManager(dsn="sqlite:///:memory:")
        dependency = create_pybatis_dependency(manager)

        async def get_user_repository(pybatis: PyBatis = Depends(dependency)) -> UserRepository:
            return UserRepository(pybatis)

        @app.get("/users/{user_id}")
        async def get_user(
            user_id: int,
            user_repo: UserRepository = Depends(get_user_repository)
        ):
            user = await user_repo.get_user(user_id)
            return user.model_dump()

        client = TestClient(app)
        response = client.get("/users/1")

        assert response.status_code == 200
        assert response.json() == {
            "id": 1,
            "name": "Test User",
            "email": "test@example.com"
        }


"""
pyBatis - FastAPI를 위한 MyBatis 스타일의 SQL 매퍼

이 라이브러리는 FastAPI 백엔드 개발자를 위한 SQL 매퍼로,
Java의 MyBatis에서 영감을 받아 Pythonic한 방식으로 구현되었습니다.
"""

from .pybatis import PyBatis
from .sql_loader import SqlLoader

# FastAPI 통합 모듈은 선택적으로 import
try:
    from .fastapi import (
        PyBatisManager,
        create_pybatis_dependency,
        get_pybatis,
        transaction_context,
        startup_pybatis,
        shutdown_pybatis,
        lifespan_pybatis,
    )
    _FASTAPI_AVAILABLE = True
except ImportError:
    _FASTAPI_AVAILABLE = False

__version__ = "0.1.0"
__author__ = "pyBatis Contributors"

__all__ = [
    "PyBatis",
    "SqlLoader",
]

# FastAPI가 사용 가능한 경우에만 추가
if _FASTAPI_AVAILABLE:
    __all__.extend([
        "PyBatisManager",
        "create_pybatis_dependency",
        "get_pybatis",
        "transaction_context",
        "startup_pybatis",
        "shutdown_pybatis",
        "lifespan_pybatis",
    ])



"""FastAPI 의존성 주입 통합 모듈

이 모듈은 PyBatis를 FastAPI와 통합하여 의존성 주입을 지원합니다.
주요 기능:
- PyBatis 인스턴스의 생명주기 관리
- FastAPI 의존성 주입 지원
- 트랜잭션 컨텍스트 관리
- 요청별 세션 관리
"""

import os
from contextlib import asynccontextmanager
from typing import Optional, AsyncGenerator, Callable, Awaitable

from pybatis import PyBatis


class PyBatisManager:
    """PyBatis 인스턴스의 생명주기를 관리하는 클래스

    이 클래스는 PyBatis 인스턴스를 생성하고 관리합니다.
    DSN 문자열 또는 기존 PyBatis 인스턴스를 받아서 초기화할 수 있습니다.

    Args:
        dsn: 데이터베이스 연결 문자열
        pybatis: 기존 PyBatis 인스턴스

    Raises:
        ValueError: dsn과 pybatis 중 하나만 제공되어야 함
    """

    def __init__(
        self,
        dsn: Optional[str] = None,
        pybatis: Optional[PyBatis] = None
    ) -> None:
        if dsn is None and pybatis is None:
            raise ValueError("dsn 또는 pybatis 중 하나는 반드시 제공되어야 합니다")

        if dsn is not None and pybatis is not None:
            raise ValueError("dsn과 pybatis는 동시에 제공될 수 없습니다")

        self.dsn = dsn
        self.pybatis = pybatis
        self._instance: Optional[PyBatis] = None

    async def get_pybatis(self) -> PyBatis:
        """PyBatis 인스턴스를 반환합니다.

        DSN이 제공된 경우 새로운 인스턴스를 생성하고,
        기존 인스턴스가 제공된 경우 그것을 반환합니다.

        Returns:
            PyBatis 인스턴스
        """
        if self.pybatis is not None:
            return self.pybatis

        if self._instance is None:
            self._instance = PyBatis(self.dsn)

        return self._instance

    async def close(self) -> None:
        """리소스를 정리합니다.

        생성된 PyBatis 인스턴스가 있다면 close 메서드를 호출합니다.
        """
        if self._instance is not None:
            await self._instance.close()
            self._instance = None


def create_pybatis_dependency(manager: PyBatisManager) -> Callable[[], Awaitable[PyBatis]]:
    """PyBatis 의존성 함수를 생성합니다.

    이 함수는 FastAPI의 Depends와 함께 사용할 수 있는 의존성 함수를 생성합니다.

    Args:
        manager: PyBatis 매니저 인스턴스

    Returns:
        FastAPI 의존성으로 사용할 수 있는 함수

    Example:
        ```python
        from fastapi import FastAPI, Depends
        from pybatis.fastapi import PyBatisManager, create_pybatis_dependency

        app = FastAPI()
        manager = PyBatisManager(dsn="sqlite:///:memory:")
        get_pybatis = create_pybatis_dependency(manager)

        @app.get("/users/{user_id}")
        async def get_user(user_id: int, pybatis: PyBatis = Depends(get_pybatis)):
            return await pybatis.fetch_one("SELECT * FROM users WHERE id = ?", user_id)
        ```
    """
    async def dependency() -> PyBatis:
        return await manager.get_pybatis()

    return dependency


# 기본 매니저 인스턴스 (환경변수 기반)
_default_manager: Optional[PyBatisManager] = None


async def get_pybatis() -> PyBatis:
    """기본 PyBatis 의존성 함수

    DATABASE_URL 환경변수를 사용하여 PyBatis 인스턴스를 생성합니다.
    이 함수는 FastAPI의 Depends와 함께 사용할 수 있습니다.

    Returns:
        PyBatis 인스턴스

    Raises:
        ValueError: DATABASE_URL 환경변수가 설정되지 않은 경우

    Example:
        ```python
        from fastapi import FastAPI, Depends
        from pybatis.fastapi import get_pybatis

        app = FastAPI()

        @app.get("/users/{user_id}")
        async def get_user(user_id: int, pybatis: PyBatis = Depends(get_pybatis)):
            return await pybatis.fetch_one("SELECT * FROM users WHERE id = ?", user_id)
        ```
    """
    global _default_manager

    if _default_manager is None:
        database_url = os.getenv("DATABASE_URL")
        if database_url is None:
            raise ValueError("DATABASE_URL 환경변수가 설정되지 않았습니다")

        _default_manager = PyBatisManager(dsn=database_url)

    return await _default_manager.get_pybatis()


@asynccontextmanager
async def transaction_context(pybatis: PyBatis) -> AsyncGenerator[PyBatis, None]:
    """트랜잭션 컨텍스트 매니저

    이 함수는 PyBatis의 트랜잭션 기능을 async with 문과 함께 사용할 수 있게 해줍니다.
    트랜잭션 내에서 예외가 발생하면 자동으로 롤백되고,
    정상적으로 완료되면 커밋됩니다.

    Args:
        pybatis: PyBatis 인스턴스

    Yields:
        트랜잭션이 활성화된 PyBatis 인스턴스

    Example:
        ```python
        from fastapi import FastAPI, Depends
        from pybatis.fastapi import get_pybatis, transaction_context

        app = FastAPI()

        @app.post("/users")
        async def create_user(
            user_data: dict,
            pybatis: PyBatis = Depends(get_pybatis)
        ):
            async with transaction_context(pybatis) as tx:
                user_id = await tx.execute(
                    "INSERT INTO users (name, email) VALUES (?, ?)",
                    user_data["name"], user_data["email"]
                )
                await tx.execute(
                    "INSERT INTO user_profiles (user_id, created_at) VALUES (?, ?)",
                    user_id, datetime.now()
                )
                return {"id": user_id, "created": True}
        ```
    """
    async with pybatis.transaction() as tx:
        yield tx


# 애플리케이션 생명주기 관리를 위한 헬퍼 함수들

async def startup_pybatis(manager: Optional[PyBatisManager] = None) -> None:
    """애플리케이션 시작 시 PyBatis 초기화

    FastAPI의 lifespan 이벤트와 함께 사용할 수 있습니다.

    Args:
        manager: 사용할 PyBatis 매니저 (None인 경우 기본 매니저 사용)
    """
    if manager is None:
        # 기본 매니저 초기화를 위해 get_pybatis 호출
        await get_pybatis()
    else:
        await manager.get_pybatis()


async def shutdown_pybatis(manager: Optional[PyBatisManager] = None) -> None:
    """애플리케이션 종료 시 PyBatis 리소스 정리

    FastAPI의 lifespan 이벤트와 함께 사용할 수 있습니다.

    Args:
        manager: 정리할 PyBatis 매니저 (None인 경우 기본 매니저 사용)
    """
    global _default_manager

    if manager is None:
        if _default_manager is not None:
            await _default_manager.close()
            _default_manager = None
    else:
        await manager.close()


@asynccontextmanager
async def lifespan_pybatis(
    app,
    manager: Optional[PyBatisManager] = None
) -> AsyncGenerator[None, None]:
    """FastAPI 애플리케이션의 lifespan 컨텍스트 매니저

    애플리케이션 시작 시 PyBatis를 초기화하고,
    종료 시 리소스를 정리합니다.

    Args:
        app: FastAPI 애플리케이션 인스턴스
        manager: 사용할 PyBatis 매니저 (None인 경우 기본 매니저 사용)

    Example:
        ```python
        from contextlib import asynccontextmanager
        from fastapi import FastAPI
        from pybatis.fastapi import lifespan_pybatis, PyBatisManager

        manager = PyBatisManager(dsn="sqlite:///:memory:")

        @asynccontextmanager
        async def lifespan(app: FastAPI):
            async with lifespan_pybatis(app, manager):
                yield

        app = FastAPI(lifespan=lifespan)
        ```
    """
    # 시작 시 초기화
    await startup_pybatis(manager)

    try:
        yield
    finally:
        # 종료 시 정리
        await shutdown_pybatis(manager)


"""
쿼리 로깅 및 모니터링 기능 테스트

이 모듈은 PyBatis의 쿼리 로깅, 실행 시간 측정, 성능 모니터링 기능을 테스트합니다.
"""

import asyncio
import logging
import time
from typing import List, Dict, Any
from unittest.mock import Mock, patch

import pytest

from pybatis import PyBatis
from tests.fixtures import MockConnection


class TestQueryLogging:
    """쿼리 로깅 기능 테스트"""

    @pytest.fixture
    def mock_connection(self):
        """테스트용 연결 픽스처"""
        return MockConnection()

    @pytest.fixture
    def pybatis_with_logging(self, mock_connection):
        """로깅이 활성화된 PyBatis 인스턴스"""
        db = PyBatis()
        db._connection = mock_connection
        db.enable_query_logging(level=logging.DEBUG)
        return db

    @pytest.mark.asyncio
    async def test_query_logging_enabled(self, pybatis_with_logging, mock_connection):
        """쿼리 로깅이 활성화되었을 때 로그가 기록되는지 테스트"""
        with patch('pybatis.pybatis.logger') as mock_logger:
            sql = "SELECT * FROM users WHERE id = :user_id"
            params = {"user_id": 1}

            await pybatis_with_logging.fetch_one(sql, params)

            # 로그가 기록되었는지 확인 (log 메서드 호출 확인)
            mock_logger.log.assert_called()
            call_args = mock_logger.log.call_args_list

            # SQL 실행 로그 확인
            assert any("SQL 실행" in str(call) for call in call_args)

    @pytest.mark.asyncio
    async def test_query_logging_disabled(self, mock_connection):
        """쿼리 로깅이 비활성화되었을 때 로그가 기록되지 않는지 테스트"""
        db = PyBatis()
        db._connection = mock_connection
        # 로깅 비활성화 (기본값)

        with patch('pybatis.pybatis.logger') as mock_logger:
            sql = "SELECT * FROM users WHERE id = :user_id"
            params = {"user_id": 1}

            await db.fetch_one(sql, params)

            # 쿼리 실행 로그가 기록되지 않았는지 확인
            log_calls = [call for call in mock_logger.log.call_args_list
                        if "SQL 실행" in str(call)]
            assert len(log_calls) == 0

    @pytest.mark.asyncio
    async def test_query_logging_with_execution_time(self, pybatis_with_logging, mock_connection):
        """쿼리 실행 시간이 로그에 포함되는지 테스트"""
        # MockConnection에 지연 시간 추가
        async def slow_fetchrow(sql, params):
            await asyncio.sleep(0.1)  # 100ms 지연
            return {"id": 1, "name": "테스트사용자"}

        mock_connection.fetchrow = slow_fetchrow

        with patch('pybatis.pybatis.logger') as mock_logger:
            sql = "SELECT * FROM users WHERE id = :user_id"
            params = {"user_id": 1}

            await pybatis_with_logging.fetch_one(sql, params)

            # 실행 시간이 포함된 로그 확인
            call_args = mock_logger.log.call_args_list
            execution_time_logs = [call for call in call_args
                                 if "실행 시간" in str(call)]
            assert len(execution_time_logs) > 0

    @pytest.mark.asyncio
    async def test_query_logging_with_parameters(self, pybatis_with_logging, mock_connection):
        """쿼리 파라미터가 로그에 포함되는지 테스트"""
        with patch('pybatis.pybatis.logger') as mock_logger:
            sql = "SELECT * FROM users WHERE id = :user_id AND name = :name"
            params = {"user_id": 1, "name": "테스트사용자"}

            await pybatis_with_logging.fetch_one(sql, params)

            # 파라미터가 포함된 로그 확인
            call_args = mock_logger.log.call_args_list
            param_logs = [call for call in call_args
                         if "params" in str(call)]
            assert len(param_logs) > 0

    @pytest.mark.asyncio
    async def test_query_logging_different_methods(self, pybatis_with_logging, mock_connection):
        """다양한 쿼리 메서드에서 로깅이 작동하는지 테스트"""
        with patch('pybatis.pybatis.logger') as mock_logger:
            # fetch_val 테스트
            await pybatis_with_logging.fetch_val("SELECT COUNT(*) FROM users")

            # fetch_all 테스트
            await pybatis_with_logging.fetch_all("SELECT * FROM users")

            # execute 테스트
            await pybatis_with_logging.execute("INSERT INTO users (name) VALUES (:name)",
                                             {"name": "새사용자"})

            # 각 메서드에 대한 로그가 기록되었는지 확인
            call_args = mock_logger.log.call_args_list
            assert len(call_args) >= 3  # 최소 3개의 로그 호출


class TestQueryMonitoring:
    """쿼리 모니터링 기능 테스트"""

    @pytest.fixture
    def mock_connection(self):
        """테스트용 연결 픽스처"""
        return MockConnection()

    @pytest.fixture
    def pybatis_with_monitoring(self, mock_connection):
        """모니터링이 활성화된 PyBatis 인스턴스"""
        db = PyBatis()
        db._connection = mock_connection
        db.enable_query_monitoring()
        return db

    @pytest.mark.asyncio
    async def test_query_monitoring_enabled(self, pybatis_with_monitoring, mock_connection):
        """쿼리 모니터링이 활성화되었을 때 통계가 수집되는지 테스트"""
        sql = "SELECT * FROM users WHERE id = :user_id"
        params = {"user_id": 1}

        await pybatis_with_monitoring.fetch_one(sql, params)

        # 쿼리 통계 확인
        stats = pybatis_with_monitoring.get_query_stats()
        assert stats is not None
        assert stats["total_queries"] == 1
        assert "fetch_one" in stats["method_counts"]

    @pytest.mark.asyncio
    async def test_query_monitoring_multiple_queries(self, pybatis_with_monitoring, mock_connection):
        """여러 쿼리 실행 시 통계가 누적되는지 테스트"""
        # 여러 쿼리 실행
        await pybatis_with_monitoring.fetch_val("SELECT COUNT(*) FROM users")
        await pybatis_with_monitoring.fetch_one("SELECT * FROM users WHERE id = 1")
        await pybatis_with_monitoring.fetch_all("SELECT * FROM users")
        await pybatis_with_monitoring.execute("INSERT INTO users (name) VALUES ('test')")

        stats = pybatis_with_monitoring.get_query_stats()
        assert stats["total_queries"] == 4
        assert stats["method_counts"]["fetch_val"] == 1
        assert stats["method_counts"]["fetch_one"] == 1
        assert stats["method_counts"]["fetch_all"] == 1
        assert stats["method_counts"]["execute"] == 1

    @pytest.mark.asyncio
    async def test_query_monitoring_execution_times(self, pybatis_with_monitoring, mock_connection):
        """쿼리 실행 시간이 기록되는지 테스트"""
        # MockConnection에 지연 시간 추가
        async def slow_fetchval(sql, params):
            await asyncio.sleep(0.05)  # 50ms 지연
            return 1

        mock_connection.fetchval = slow_fetchval

        await pybatis_with_monitoring.fetch_val("SELECT COUNT(*) FROM users")

        stats = pybatis_with_monitoring.get_query_stats()
        assert "execution_times" in stats
        assert len(stats["execution_times"]) == 1
        assert stats["execution_times"][0] >= 0.05  # 최소 50ms

    @pytest.mark.asyncio
    async def test_query_monitoring_slow_query_detection(self, pybatis_with_monitoring, mock_connection):
        """느린 쿼리 감지 기능 테스트"""
        # 로깅도 활성화 (느린 쿼리 경고를 위해)
        pybatis_with_monitoring.enable_query_logging()

        # 느린 쿼리 임계값 설정 (100ms)
        pybatis_with_monitoring.set_slow_query_threshold(0.1)

        # MockConnection에 긴 지연 시간 추가
        async def very_slow_fetchval(sql, params):
            await asyncio.sleep(0.15)  # 150ms 지연
            return 1

        mock_connection.fetchval = very_slow_fetchval

        with patch('pybatis.pybatis.logger') as mock_logger:
            await pybatis_with_monitoring.fetch_val("SELECT COUNT(*) FROM users")

            # 느린 쿼리 경고 로그 확인
            warning_calls = [call for call in mock_logger.warning.call_args_list
                           if "느린 쿼리" in str(call)]
            assert len(warning_calls) > 0

    @pytest.mark.asyncio
    async def test_query_monitoring_reset_stats(self, pybatis_with_monitoring, mock_connection):
        """쿼리 통계 초기화 기능 테스트"""
        # 몇 개의 쿼리 실행
        await pybatis_with_monitoring.fetch_val("SELECT COUNT(*) FROM users")
        await pybatis_with_monitoring.fetch_one("SELECT * FROM users WHERE id = 1")

        # 통계 확인
        stats = pybatis_with_monitoring.get_query_stats()
        assert stats["total_queries"] == 2

        # 통계 초기화
        pybatis_with_monitoring.reset_query_stats()

        # 초기화 후 통계 확인
        stats = pybatis_with_monitoring.get_query_stats()
        assert stats["total_queries"] == 0
        assert len(stats["execution_times"]) == 0

    def test_query_monitoring_disabled_by_default(self, mock_connection):
        """기본적으로 쿼리 모니터링이 비활성화되어 있는지 테스트"""
        db = PyBatis()
        db._connection = mock_connection

        # 모니터링이 비활성화된 상태에서는 통계가 None이어야 함
        stats = db.get_query_stats()
        assert stats is None


class TestQueryLoggingIntegration:
    """쿼리 로깅과 모니터링 통합 테스트"""

    @pytest.fixture
    def mock_connection(self):
        """테스트용 연결 픽스처"""
        return MockConnection()

    @pytest.fixture
    def pybatis_full_featured(self, mock_connection):
        """로깅과 모니터링이 모두 활성화된 PyBatis 인스턴스"""
        db = PyBatis()
        db._connection = mock_connection
        db.enable_query_logging(level=logging.INFO)
        db.enable_query_monitoring()
        return db

    @pytest.mark.asyncio
    async def test_logging_and_monitoring_together(self, pybatis_full_featured, mock_connection):
        """로깅과 모니터링이 함께 작동하는지 테스트"""
        with patch('pybatis.pybatis.logger') as mock_logger:
            sql = "SELECT * FROM users WHERE id = :user_id"
            params = {"user_id": 1}

            await pybatis_full_featured.fetch_one(sql, params)

            # 로깅 확인
            assert mock_logger.log.called

            # 모니터링 확인
            stats = pybatis_full_featured.get_query_stats()
            assert stats["total_queries"] == 1

    @pytest.mark.asyncio
    async def test_transaction_logging(self, pybatis_full_featured, mock_connection):
        """트랜잭션 내에서의 로깅 테스트"""
        with patch('pybatis.pybatis.logger') as mock_logger:
            async with pybatis_full_featured.transaction() as tx:
                await tx.execute("INSERT INTO users (name) VALUES (:name)", {"name": "사용자1"})
                await tx.execute("INSERT INTO users (name) VALUES (:name)", {"name": "사용자2"})

            # 트랜잭션 관련 로그 확인
            call_args = mock_logger.log.call_args_list
            transaction_logs = [call for call in call_args
                              if "트랜잭션" in str(call) or "transaction" in str(call).lower()]
            # 트랜잭션 시작과 커밋 로그가 있어야 함
            assert len(transaction_logs) >= 2


"""
SQL 파일 로더

.sql 파일에서 SQL 문을 로드하는 기능을 제공합니다.
"""

import re
from pathlib import Path
from typing import Optional, Union


class SqlLoader:
    """
    SQL 파일 로더 클래스

    .sql 파일에서 SQL 문을 로드하고,
    이름 기반으로 특정 SQL을 추출하는 기능을 제공합니다.
    """

    def __init__(self, sql_dir: Union[str, Path]):
        """
        SQL 로더를 초기화합니다.

        Args:
            sql_dir: SQL 파일들이 있는 디렉토리 경로
        """
        self.sql_dir = Path(sql_dir)
        if not self.sql_dir.exists():
            raise ValueError(f"SQL 디렉토리가 존재하지 않습니다: {self.sql_dir}")
        if not self.sql_dir.is_dir():
            raise ValueError(f"SQL 경로가 디렉토리가 아닙니다: {self.sql_dir}")

    def load_sql(self, filename: str, name: Optional[str] = None) -> str:
        """
        SQL 파일에서 SQL 문을 로드합니다.

        Args:
            filename: SQL 파일명 (예: "users.sql")
            name: 로드할 SQL의 이름 (옵션)

        Returns:
            로드된 SQL 문

        Raises:
            FileNotFoundError: SQL 파일이 존재하지 않는 경우
            ValueError: 지정된 이름의 SQL을 찾을 수 없는 경우
        """
        sql_file = self.sql_dir / filename

        if not sql_file.exists():
            raise FileNotFoundError(f"SQL 파일이 존재하지 않습니다: {sql_file}")

        content = sql_file.read_text(encoding="utf-8")

        if name is None:
            # 이름이 지정되지 않으면 전체 파일 내용 반환
            return self._clean_sql(content)

        # 이름으로 특정 SQL 추출
        return self._extract_named_sql(content, name)

    def _extract_named_sql(self, content: str, name: str) -> str:
        """
        파일 내용에서 특정 이름의 SQL을 추출합니다.

        SQL 파일 형식:
        -- name=sql_name
        SELECT * FROM table;

        Args:
            content: SQL 파일 전체 내용
            name: 추출할 SQL의 이름

        Returns:
            추출된 SQL 문

        Raises:
            ValueError: 지정된 이름의 SQL을 찾을 수 없는 경우
        """
        # 정규식으로 이름 기반 SQL 블록 찾기
        pattern = rf"--\s*name\s*=\s*{re.escape(name)}\s*\n(.*?)(?=--\s*name\s*=|\Z)"
        match = re.search(pattern, content, re.DOTALL | re.IGNORECASE)

        if not match:
            raise ValueError(f"SQL with name '{name}' not found in file")

        sql = match.group(1).strip()
        return self._clean_sql(sql)

    def _clean_sql(self, sql: str) -> str:
        """
        SQL 문을 정리합니다 (불필요한 공백, 주석 제거).

        Args:
            sql: 원본 SQL 문

        Returns:
            정리된 SQL 문
        """
        # 빈 줄과 앞뒤 공백 제거
        lines = []
        for line in sql.split("\n"):
            line = line.strip()
            # 빈 줄과 단순 주석 줄 건너뛰기 (-- name= 형태는 제외)
            if line and not (line.startswith("--") and "name=" not in line):
                lines.append(line)

        return " ".join(lines) if lines else sql.strip()

    def list_sql_files(self) -> list[str]:
        """
        SQL 디렉토리의 모든 .sql 파일 목록을 반환합니다.

        Returns:
            SQL 파일명 목록
        """
        return [f.name for f in self.sql_dir.glob("*.sql")]

    def list_named_sqls(self, filename: str) -> list[str]:
        """
        특정 SQL 파일에서 이름이 지정된 SQL들의 목록을 반환합니다.

        Args:
            filename: SQL 파일명

        Returns:
            이름이 지정된 SQL들의 이름 목록
        """
        sql_file = self.sql_dir / filename

        if not sql_file.exists():
            raise FileNotFoundError(f"SQL 파일이 존재하지 않습니다: {sql_file}")

        content = sql_file.read_text(encoding="utf-8")

        # 정규식으로 모든 이름 찾기
        pattern = r"--\s*name\s*=\s*(\w+)"
        matches = re.findall(pattern, content, re.IGNORECASE)

        return matches
